# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-05 17:02
from __future__ import unicode_literals

import django.db.models.deletion
import filebrowser.fields
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('crm', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='TemplateSet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('organisationname', models.CharField(max_length=200, verbose_name='Name of the Organisation')),
                ('title', models.CharField(max_length=100, verbose_name='Title')),
                ('logo', filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True,
                                                            verbose_name='Logo for the PDF generation')),
                ('bankingaccountref',
                 models.CharField(blank=True, max_length=60, null=True, verbose_name='Reference to Banking Account')),
                ('addresser', models.CharField(blank=True, max_length=200, null=True, verbose_name='Addresser')),
                ('fopConfigurationFile', filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True,
                                                                            verbose_name='FOP Configuration File')),
                ('footerTextsalesorders',
                 models.TextField(blank=True, null=True, verbose_name='Footer Text On Salesorders')),
                ('headerTextsalesorders',
                 models.TextField(blank=True, null=True, verbose_name='Header Text On Salesorders')),
                ('headerTextpurchaseorders',
                 models.TextField(blank=True, null=True, verbose_name='Header Text On Purchaseorders')),
                ('footerTextpurchaseorders',
                 models.TextField(blank=True, null=True, verbose_name='Footer Text On Purchaseorders')),
                ('pagefooterleft',
                 models.CharField(blank=True, max_length=40, null=True, verbose_name='Page Footer Left')),
                ('pagefootermiddle',
                 models.CharField(blank=True, max_length=40, null=True, verbose_name='Page Footer Middle')),
            ],
            options={
                'verbose_name': 'Templateset',
                'verbose_name_plural': 'Templatesets',
            },
        ),
        migrations.CreateModel(
            name='UserExtension',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('defaultCurrency', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Currency')),
                ('defaultTemplateSet',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.TemplateSet')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User Extention',
                'verbose_name_plural': 'User Extentions',
            },
        ),
        migrations.CreateModel(
            name='UserExtensionEmailAddress',
            fields=[
                ('emailaddress_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      primary_key=True, serialize=False, to='crm.EmailAddress')),
                ('purpose', models.CharField(
                    choices=[('H', 'Private'), ('O', 'Business'), ('P', 'Mobile Private'), ('B', 'Mobile Business')],
                    max_length=1, verbose_name='Purpose')),
                ('userExtension', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                    to='djangoUserExtension.UserExtension')),
            ],
            options={
                'verbose_name': 'Email Address for User Extention',
                'verbose_name_plural': 'Email Address for User Extention',
            },
            bases=('crm.emailaddress',),
        ),
        migrations.CreateModel(
            name='UserExtensionPhoneAddress',
            fields=[
                ('phoneaddress_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      primary_key=True, serialize=False, to='crm.PhoneAddress')),
                ('purpose', models.CharField(
                    choices=[('H', 'Private'), ('O', 'Business'), ('P', 'Mobile Private'), ('B', 'Mobile Business')],
                    max_length=1, verbose_name='Purpose')),
                ('userExtension', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                    to='djangoUserExtension.UserExtension')),
            ],
            options={
                'verbose_name': 'Phonenumber for User Extention',
                'verbose_name_plural': 'Phonenumber for User Extention',
            },
            bases=('crm.phoneaddress',),
        ),
        migrations.CreateModel(
            name='UserExtensionPostalAddress',
            fields=[
                ('postaladdress_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      primary_key=True, serialize=False, to='crm.PostalAddress')),
                ('purpose', models.CharField(
                    choices=[('H', 'Private'), ('O', 'Business'), ('P', 'Mobile Private'), ('B', 'Mobile Business')],
                    max_length=1, verbose_name='Purpose')),
                ('userExtension', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                    to='djangoUserExtension.UserExtension')),
            ],
            options={
                'verbose_name': 'Postal Address for User Extention',
                'verbose_name_plural': 'Postal Address for User Extention',
            },
            bases=('crm.postaladdress',),
        ),
        migrations.CreateModel(
            name='XSLFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=100, null=True, verbose_name='Title')),
                ('xslfile', filebrowser.fields.FileBrowseField(max_length=200, verbose_name='XSL File')),
            ],
            options={
                'verbose_name': 'XSL File',
                'verbose_name_plural': 'XSL Files',
            },
        ),
        migrations.AddField(
            model_name='templateset',
            name='balancesheetXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='db_reltemplatebalancesheet', to='djangoUserExtension.XSLFile',
                                    verbose_name='XSL File for Balancesheet'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='deilveryorderXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='db_reltemplatedeliveryorder', to='djangoUserExtension.XSLFile',
                                    verbose_name='XSL File for Deilvery Order'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='invoiceXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltemplateinvoice',
                                    to='djangoUserExtension.XSLFile', verbose_name='XSL File for Invoice'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='profitLossStatementXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='db_reltemplateprofitlossstatement', to='djangoUserExtension.XSLFile',
                                    verbose_name='XSL File for Profit Loss Statement'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='purchaseconfirmationXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='db_reltemplatepurchaseconfirmation', to='djangoUserExtension.XSLFile',
                                    verbose_name='XSL File for Purchase Confirmation'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='purchaseorderXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='db_reltemplatepurchaseorder', to='djangoUserExtension.XSLFile',
                                    verbose_name='XSL File for Purchaseorder'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='quoteXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltemplatequote',
                                    to='djangoUserExtension.XSLFile', verbose_name='XSL File for Quote'),
        ),
    ]



# -*- coding: utf-8 -*-

from django import forms
from koalixcrm.crm.views.work_entry_form import WorkEntry


class BaseWorkEntryFormset(forms.BaseFormSet):
    def __init__(self, *args, **kwargs):
        super(BaseWorkEntryFormset, self).__init__(*args, **kwargs)

    @staticmethod
    def generate_initial_data(start_date, stop_date, human_resource):
        from koalixcrm.crm.reporting.work import Work
        list_of_work = Work.objects.filter(human_resource=human_resource, date__lte=stop_date, date__gte=start_date).order_by("date")
        initial = []
        for work in list_of_work:
            initial.append({'work_id': work.id,
                            'task': work.task,
                            'project': work.task.project,
                            'datetime_start': work.start_time,
                            'datetime_stop': work.stop_time,
                            'worked_hours': work.worked_hours,
                            'description': work.description})
        return initial

    @staticmethod
    def load_formset(range_selection_form, request):
        WorkEntryFormSet = forms.formset_factory(WorkEntry,
                                                 extra=1,
                                                 max_num=60,
                                                 can_delete=True,
                                                 formset=BaseWorkEntryFormset)
        from_date = range_selection_form.evaluate_pre_check_from_date()
        to_date = range_selection_form.evaluate_pre_check_to_date()
        form_kwargs = BaseWorkEntryFormset.compose_form_kwargs(from_date, to_date)
        pre_check_formset = WorkEntryFormSet(request.POST,
                                             form_kwargs=form_kwargs)
        return pre_check_formset

    @staticmethod
    def compose_form_kwargs(from_date, to_date):
        form_kwargs = {'from_date': from_date, 'to_date': to_date}
        return form_kwargs

    @staticmethod
    def create_updated_formset(range_selection_form, human_resource):
        WorkEntryFormSet = forms.formset_factory(WorkEntry,
                                                 extra=1,
                                                 max_num=60,
                                                 can_delete=True,
                                                 formset=BaseWorkEntryFormset)
        from_date = range_selection_form.cleaned_data['from_date']
        to_date = range_selection_form.cleaned_data['to_date']
        initial_formset_data = BaseWorkEntryFormset.generate_initial_data(from_date,
                                                                          to_date,
                                                                          human_resource)
        form_kwargs = BaseWorkEntryFormset.compose_form_kwargs(from_date, to_date)
        formset = WorkEntryFormSet(initial=initial_formset_data,
                                   form_kwargs=form_kwargs)
        return formset

    @staticmethod
    def create_new_formset(from_date, to_date, human_resource):
        WorkEntryFormSet = forms.formset_factory(WorkEntry,
                                                 extra=1,
                                                 max_num=60,
                                                 can_delete=True,
                                                 formset=BaseWorkEntryFormset)
        initial_formset_data = BaseWorkEntryFormset.generate_initial_data(from_date,
                                                                          to_date,
                                                                          human_resource)
        form_kwargs = BaseWorkEntryFormset.compose_form_kwargs(from_date,
                                                               to_date)
        formset = WorkEntryFormSet(initial=initial_formset_data,
                                   form_kwargs=form_kwargs)
        return formset



from rest_framework import serializers

from koalixcrm.accounting.accounting.product_category import ProductCategory
from koalixcrm.accounting.rest.product_categorie_rest import ProductCategoryMinimalJSONSerializer
from koalixcrm.crm.product.product_type import ProductType
from koalixcrm.crm.product.tax import Tax
from koalixcrm.crm.product.unit import Unit
from koalixcrm.crm.rest.tax_rest import OptionTaxJSONSerializer
from koalixcrm.crm.rest.unit_rest import OptionUnitJSONSerializer


class ProductJSONSerializer(serializers.HyperlinkedModelSerializer):
    productNumber = serializers.IntegerField(source='product_number', allow_null=False)
    unit = OptionUnitJSONSerializer(source='default_unit', allow_null=False)
    tax = OptionTaxJSONSerializer(allow_null=False)
    productCategory = ProductCategoryMinimalJSONSerializer(source='accounting_product_categorie', allow_null=False)

    class Meta:
        model = ProductType
        fields = ('id',
                  'productNumber',
                  'title',
                  'unit',
                  'tax',
                  'productCategory')
        depth = 1

    def create(self, validated_data):
        product = ProductType()
        product.product_number = validated_data['product_number']
        product.title = validated_data['title']

        # Deserialize default_unit
        default_unit = validated_data.pop('default_unit')
        if default_unit:
            if default_unit.get('id', None):
                product.default_unit = Unit.objects.get(id=default_unit.get('id', None))
            else:
                product.default_unit = None

        # Deserialize tax
        tax = validated_data.pop('tax')
        if tax:
            if tax.get('id', None):
                product.tax = Tax.objects.get(id=tax.get('id', None))
            else:
                product.tax = None

        # Deserialize product category
        product_category = validated_data.pop('accounting_product_categorie')
        if product_category:
            if product_category.get('id', None):
                product.accounting_product_category = ProductCategory.objects.get(id=product_category.get('id', None))
            else:
                product.accounting_product_category = None

        product.save()
        return product

    def update(self, instance, validated_data):
        instance.title = validated_data['title']
        instance.product_number = validated_data['product_number']

        # Deserialize default_unit
        default_unit = validated_data.pop('default_unit')
        if default_unit:
            if default_unit.get('id', instance.default_unit):
                instance.default_unit = Unit.objects.get(id=default_unit.get('id', None))
            else:
                instance.default_unit = instance.default_unit
        else:
            instance.default_unit = None

        # Deserialize tax
        tax = validated_data.pop('tax')
        if tax:
            if tax.get('id', instance.default_unit):
                instance.tax = Tax.objects.get(id=tax.get('id', None))
            else:
                instance.tax = instance.tax
        else:
            instance.tax = None

        # Deserialize product category
        product_category = validated_data.pop('accounting_product_categorie')
        if product_category:
            if product_category.get('id', instance.accounting_product_categorie):
                instance.accounting_product_categorie = ProductCategory.objects.get(
                    id=product_category.get('id', None))
            else:
                instance.accounting_product_categorie = instance.accounting_product_categorie
        else:
            instance.accounting_product_categorie = None

        instance.save()
        return instance



# -*- coding: utf-8 -*-
import pytest
from django.test import LiveServerTestCase
from selenium import webdriver
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from koalixcrm.crm.factories.factory_user import AdminUserFactory


class TestProjectAdminView(LiveServerTestCase):

    def setUp(self):
        firefox_options = webdriver.firefox.options.Options()
        firefox_options.set_headless(headless=True)
        self.selenium = webdriver.Firefox(firefox_options=firefox_options)
        self.test_user = AdminUserFactory.create()

    def tearDown(self):
        self.selenium.quit()

    @pytest.mark.front_end_tests
    def test_project_admin(self):
        selenium = self.selenium
        # login
        selenium.get('%s%s' % (self.live_server_url, '/admin/crm/project/'))
        # the browser will be redirected to the login page
        timeout = 5
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, 'id_username'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")
        username = selenium.find_element_by_xpath('//*[@id="id_username"]')
        password = selenium.find_element_by_xpath('//*[@id="id_password"]')
        submit_button = selenium.find_element_by_xpath('/html/body/div/article/div/div/form/div/ul/li/input')
        username.send_keys("admin")
        password.send_keys("admin")
        submit_button.send_keys(Keys.RETURN)
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID,
                                                                               '/html/body/div/article/header/ul/li/a'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")
        # find the form element
        selenium.get('%s%s' % (self.live_server_url, '/admin/crm/project/add'))

        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, '//*[@id="id_project_status"]'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")



from rest_framework import serializers

from koalixcrm.accounting.accounting.account import Account
from koalixcrm.accounting.rest.account_rest import OptionAccountJSONSerializer
from koalixcrm.crm.product.tax import Tax


class OptionTaxJSONSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(required=False)
    description = serializers.CharField(source='name', read_only=True)

    class Meta:
        model = Tax
        fields = ('id',
                  'description')


class TaxJSONSerializer(serializers.HyperlinkedModelSerializer):
    rate = serializers.CharField(source='tax_rate')
    description = serializers.CharField(source='name')
    assetAccount = OptionAccountJSONSerializer(source='account_activa', allow_null=True)
    liabilityAccount = OptionAccountJSONSerializer(source='account_passiva', allow_null=True)

    class Meta:
        model = Tax
        fields = ('id',
                  'rate',
                  'description',
                  'assetAccount',
                  'liabilityAccount')

    def create(self, validated_data):
        tax = Tax()
        tax.tax_rate = validated_data['tax_rate']
        tax.name = validated_data['name']

        # Deserialize account activa
        account_activa = validated_data.pop('account_activa')
        if account_activa:
            if account_activa.get('id', None):
                tax.account_activa = Account.objects.get(id=account_activa.get('id', None))
            else:
                tax.account_activa = None

        # Deserialize account passiva
        account_passiva = validated_data.pop('account_passiva')
        if account_passiva:
            if account_passiva.get('id', None):
                tax.account_passiva = Account.objects.get(id=account_passiva.get('id', None))
            else:
                tax.account_passiva = None

        tax.save()
        return tax

    def update(self, instance, validated_data):
        instance.tax_rate = validated_data['tax_rate']
        instance.name = validated_data['name']

        # Deserialize account activa
        account_activa = validated_data.pop('account_activa')
        if account_activa:
            if account_activa.get('id', instance.account_activa):
                instance.account_activa = Account.objects.get(id=account_activa.get('id', None))
            else:
                instance.account_activa = instance.account_activa_id
        else:
            instance.account_activa = None

        # Deserialize account passiva
        account_passiva = validated_data.pop('account_passiva')
        if account_passiva:
            if account_passiva.get('id', instance.account_passiva):
                instance.account_passiva = Account.objects.get(id=account_passiva.get('id', None))
            else:
                instance.account_passiva = instance.account_passiva_id
        else:
            instance.account_passiva = None

        instance.save()
        return instance



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-19 21:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0022_auto_20180117_2020'),
    ]

    operations = [
        migrations.AlterField(
            model_name='position',
            name='position_number',
            field=models.PositiveIntegerField(verbose_name='Position Number'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib import admin
from django.db import models
from django.utils.translation import ugettext as _

from koalixcrm.crm.product.product_price import ProductPrice
from koalixcrm.crm.product.product_price import ProductPriceInlineAdminView
from koalixcrm.crm.product.unit_transform import UnitTransformInlineAdminView
from koalixcrm.crm.product.customer_group_transform import CustomerGroupTransformInlineAdminView
from koalixcrm.crm.product.currency_transform import CurrencyTransformInlineAdminView


class ProductType(models.Model):
    description = models.TextField(verbose_name=_("Description"),
                                   null=True,
                                   blank=True)
    title = models.CharField(verbose_name=_("Title"),
                             max_length=200)
    product_type_identifier = models.CharField(verbose_name=_("Product Number"),
                                               max_length=200,
                                               null=True,
                                               blank=True)
    default_unit = models.ForeignKey("Unit", on_delete=models.CASCADE, verbose_name=_("Unit"))
    tax = models.ForeignKey("Tax",
                            on_delete=models.CASCADE,
                            blank=False,
                            null=False)
    last_modification = models.DateTimeField(verbose_name=_("Last modified"),
                                             auto_now=True)
    last_modified_by = models.ForeignKey('auth.User',
                                         on_delete=models.CASCADE,
                                         limit_choices_to={'is_staff': True},
                                         verbose_name=_("Last modified by"),
                                         null=True,
                                         blank=True)
    date_of_creation = models.DateTimeField(verbose_name=_("Created at"),
                                            auto_now_add=True)
    accounting_product_category = models.ForeignKey('accounting.ProductCategory',
                                                    on_delete=models.CASCADE,
                                                    verbose_name=_("Accounting Product Category"),
                                                    null=True,
                                                    blank=True)

    def get_price(self, date, unit, customer, currency):
        """The function searches for a valid price and returns the price of the product as a decimal value.

        Args:
            koalixcrm.crm.contact.customer customer
            koalixcrm.crm.product.unit unit
            koalixcrm.crm.product.currency currency
            datetime.date date

        Returns:
            when a match is found: dict customer_group_factors name=customer_group, value=factor
            when no match is found: customer_group_factors is None

        Raises:
            In case the algorithm does not find a valid product price, the function raises a
            NoPriceFound Exception"""
        prices = ProductPrice.objects.filter(product_type=self)
        valid_prices = list()
        for price in list(prices):
            currency_factor = price.get_currency_transform_factor(currency, self.id)
            unit_factor = price.get_unit_transform_factor(unit, self.id)
            group_factor = price.get_customer_group_transform_factor(customer, self.id)
            date_in_range = price.is_date_in_range(date)
            if date_in_range \
                    and currency_factor != 0 \
                    and unit_factor != 0 \
                    and group_factor != 0:
                transformed_price = price.price*group_factor*unit_factor*currency_factor
                valid_prices.append(transformed_price)
        if len(valid_prices) > 0:
            lowest_price = valid_prices[0]
            for price in valid_prices:
                if price < lowest_price:
                    lowest_price = price
            return lowest_price
        else:
            raise ProductType.NoPriceFound(customer, unit, date, currency, self)

    def get_tax_rate(self):
        return self.tax.get_tax_rate()

    def __str__(self):
        return str(self.product_type_identifier) + ' ' + self.title.__str__()

    class Meta:
        app_label = "crm"
        verbose_name = _('Product Type')
        verbose_name_plural = _('Product Types')

    class NoPriceFound(Exception):
        def __init__(self, customer, unit, date, currency, product):
            self.customer = customer
            self.unit = unit
            self.date = date
            self.product = product
            self.currency = currency
            return

        def __str__(self):
            return _("There is no Price for this product type") + ": " + self.product.__str__() + _(
                "that matches the date") + ": " + self.date.__str__() + " ," + _(
                "customer") + ": " + self.customer.__str__() + " ," + _(
                "currency") + ": " + self.currency.__str__() + _(" and unit") + ":" + self.unit.__str__()


class ProductTypeAdminView(admin.ModelAdmin):
    list_display = (
        'product_type_identifier',
        'title',
        'default_unit',
        'tax',
        'accounting_product_category')
    list_display_links = ('product_type_identifier',)
    fieldsets = (
        (_('Basics'), {
            'fields': (
                'product_type_identifier',
                'title',
                'description',
                'default_unit',
                'tax',
                'accounting_product_category')
        }),
    )
    inlines = [ProductPriceInlineAdminView,
               UnitTransformInlineAdminView,
               CurrencyTransformInlineAdminView,
               CustomerGroupTransformInlineAdminView]



from rest_framework import serializers

from koalixcrm.djangoUserExtension.user_extension.document_template import InvoiceTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import QuoteTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import DeliveryNoteTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import PaymentReminderTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import PurchaseOrderTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import PurchaseConfirmationTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import ProfitLossStatementTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import BalanceSheetTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import MonthlyProjectSummaryTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import WorkReportTemplate


class OptionDocumentTemplateJSONSerializer(serializers.HyperlinkedModelSerializer):
    title = serializers.CharField(required=False, read_only=True)
    xslFile = serializers.FileField(source='xsl_file', read_only=True)
    fopConfigFile = serializers.FileField(source='user', read_only=True)
    logo = serializers.FileField(read_only=True)


class OptionInvoiceTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = InvoiceTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionQuoteTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = QuoteTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionDeliveryNoteTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = DeliveryNoteTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionPaymentReminderTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = PaymentReminderTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionPurchaseOrderTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = PurchaseOrderTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionPurchaseConfirmationTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = PurchaseConfirmationTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionProfitLossStatementTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = ProfitLossStatementTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionBalanceSheetTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = BalanceSheetTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionMonthlyProjectSummaryTemplateTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = MonthlyProjectSummaryTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')


class OptionWorkReportTemplateJSONSerializer(OptionDocumentTemplateJSONSerializer):
    class Meta:
        model = WorkReportTemplate
        fields = ('title',
                  'xslFile',
                  'fopConfigFile',
                  'logo')



"""
Django settings for test_koalixcrm project.

Generated by 'django-admin startproject' using Django 1.11.3.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""
from .base_settings import *

# Update database configuration with $DATABASE_URL.
import dj_database_url
DATABASES = { 'default': dj_database_url.config(conn_max_age=500) }
FOP_EXECUTABLE = "/usr/bin/fop"
GRAPPELLI_INDEX_DASHBOARD = 'projectsettings.dashboard.CustomIndexDashboard'
ALLOWED_HOSTS = ['koalix-crm.herokuapp.com']

KOALIXCRM_REST_API_AUTH = True



# -*- coding: utf-8 -*

from django.utils.translation import ugettext as _

PURPOSESADDRESSINUSEREXTENTION = (
    ('H', _('Private')),
    ('O', _('Business')),
    ('P', _('Mobile Private')),
    ('B', _('Mobile Business')),
)



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _


class PhoneAddress(models.Model):
    phone = models.CharField(max_length=20,
                             verbose_name=_("Phone Number"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address')
        verbose_name_plural = _('Phone Address')



# -*- coding: utf-8 -*-

from rest_framework import serializers

from koalixcrm.accounting.accounting.accounting_period import AccountingPeriod
from koalixcrm.accounting.models import Account


class OptionAccountingPeriodJSONSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(required=False)
    title = serializers.CharField(read_only=True)

    class Meta:
        model = Account
        fields = ('id',
                  'title')


class AccountingPeriodJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = AccountingPeriod
        fields = ('id',
                  'title',
                  'begin',
                  'end')
        depth = 1



from setuptools import setup, find_packages
from koalixcrm.version import KOALIXCRM_VERSION

setup(name='koalix-crm',
      version=KOALIXCRM_VERSION,
      description='koalixcrm is a tiny and easy to use Customer-Relationship-Management'
                  ' Software (CRM) including tiny and easy to use Accounting Software',
      url='http://github.com/scaphilo/koalixcrm',
      author='Aaron Riedener',
      author_email='aaron.riedener@gmail.com',
      license='BSD',
      packages=find_packages(exclude=["projectsettings", "documentation"]),
      install_requires=['Django==2.2.9',
                        'django-filebrowser==3.12.1',
                        'lxml==4.4.2',
                        'olefile==0.46',
                        'Pillow==6.2.0',
                        'psycopg2-binary==2.8.4',
                        'pytz==2019.3',
                        'django-grappelli==2.13.3',
                        'djangorestframework==3.11.0',
                        'djangorestframework-xml==1.4.0',
                        'markdown==3.1.1',
                        'django-filter==2.2.0',
                        'pandas==0.25.2',
                        'matplotlib==3.1.1',
                        ],
      zip_safe=False,
      classifiers=['Development Status :: 4 - Beta',
                   'Programming Language :: Python :: 3.4', ],
      python_requires='~=3.5',
      include_package_data=True,
)



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import EstimationStatus


class ObsoleteEstimationStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = EstimationStatus
        django_get_or_create = ('title',)

    title = "Done"
    description = "This estimation is obsolete"
    is_obsolete = True


class PlannedEstimationStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = EstimationStatus
        django_get_or_create = ('title',)

    title = "Planned"
    description = "This estimation is planned"
    is_obsolete = False


class StartedEstimationStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = EstimationStatus
        django_get_or_create = ('title',)

    title = "Started"
    description = "This estimation is started"
    is_obsolete = False



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import ReportingPeriod
from koalixcrm.crm.factories.factory_project import StandardProjectFactory
from koalixcrm.crm.factories.factory_reporting_period_status import ReportingReportingPeriodStatusFactory


class StandardReportingPeriodFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ReportingPeriod
        django_get_or_create = ('title',)

    project = factory.SubFactory(StandardProjectFactory)
    title = "This is a test project"
    begin = '2018-06-15'
    end = '2044-06-15'
    status = factory.SubFactory(ReportingReportingPeriodStatusFactory)



# -*- coding: utf-8 -*-

from .base_settings import *

# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases


ALLOWED_HOSTS = ['*']

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

FOP_EXECUTABLE = "C:/Users/ria/Downloads/fop-2.3/fop/fop.cmd"
GRAPPELLI_INDEX_DASHBOARD = 'projectsettings.dashboard.CustomIndexDashboard'

KOALIXCRM_REST_API_AUTH = True



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import PostalAddress


class StandardPostalAddressFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = PostalAddress

    prefix = "M"
    name = "Smith"
    pre_name = "John"
    address_line_1 = "Main-street 5"
    address_line_2 = None
    address_line_3 = None
    address_line_4 = None
    zip_code = 8000
    town = "Zürich"
    state = "ZH"
    country = "CH"



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-05 17:02
from __future__ import unicode_literals

import django.db.models.deletion
import filebrowser.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('crm', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contract', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Contract',
                                               verbose_name='Subscription Type')),
            ],
            options={
                'verbose_name': 'Subscription',
                'verbose_name_plural': 'Subscriptions',
            },
        ),
        migrations.CreateModel(
            name='SubscriptionEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('eventdate', models.DateField(blank=True, null=True, verbose_name='Event Date')),
                ('event', models.CharField(choices=[('O', 'Offered'), ('C', 'Canceled'), ('S', 'Signed')], max_length=1,
                                           verbose_name='Event')),
                ('subscriptions',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subscriptions.Subscription',
                                   verbose_name='Subscription')),
            ],
            options={
                'verbose_name': 'Subscription Event',
                'verbose_name_plural': 'Subscription Events',
            },
        ),
        migrations.CreateModel(
            name='SubscriptionType',
            fields=[
                ('product_ptr',
                 models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True,
                                      primary_key=True, serialize=False, to='crm.Product')),
                ('cancelationPeriod',
                 models.IntegerField(blank=True, null=True, verbose_name='Cancelation Period (months)')),
                ('automaticContractExtension',
                 models.IntegerField(blank=True, null=True, verbose_name='Automatic Contract Extension (months)')),
                ('automaticContractExtensionReminder', models.IntegerField(blank=True, null=True,
                                                                           verbose_name='Automatic Contract Extensoin Reminder (days)')),
                ('minimumDuration',
                 models.IntegerField(blank=True, null=True, verbose_name='Minimum Contract Duration')),
                ('paymentIntervall',
                 models.IntegerField(blank=True, null=True, verbose_name='Payment Intervall (days)')),
                ('contractDocument', filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True,
                                                                        verbose_name='Contract Documents')),
            ],
            options={
                'verbose_name': 'Subscription Type',
                'verbose_name_plural': 'Subscription Types',
            },
            bases=('crm.product',),
        ),
        migrations.AddField(
            model_name='subscription',
            name='subscriptiontype',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                    to='subscriptions.SubscriptionType', verbose_name='Subscription Type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-02 16:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0037_auto_20180701_2128'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='postaladdress',
            name='addressline1',
        ),
        migrations.RemoveField(
            model_name='postaladdress',
            name='addressline2',
        ),
        migrations.RemoveField(
            model_name='postaladdress',
            name='addressline3',
        ),
        migrations.RemoveField(
            model_name='postaladdress',
            name='addressline4',
        ),
        migrations.RemoveField(
            model_name='postaladdress',
            name='prename',
        ),
        migrations.RemoveField(
            model_name='postaladdress',
            name='zipcode',
        ),
        migrations.AddField(
            model_name='postaladdress',
            name='address_line_1',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Address line 1'),
        ),
        migrations.AddField(
            model_name='postaladdress',
            name='address_line_2',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Address line 2'),
        ),
        migrations.AddField(
            model_name='postaladdress',
            name='address_line_3',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Address line 3'),
        ),
        migrations.AddField(
            model_name='postaladdress',
            name='address_line_4',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Address line 4'),
        ),
        migrations.AddField(
            model_name='postaladdress',
            name='pre_name',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Pre-name'),
        ),
        migrations.AddField(
            model_name='postaladdress',
            name='zip_code',
            field=models.IntegerField(blank=True, null=True, verbose_name='Zip Code'),
        ),
    ]



class LimitedAdminInlineMixin(object):
    """
    InlineAdmin mixin limiting the selection of related items according to
    criteria which can depend on the current parent object being edited.

    A typical use case would be selecting a subset of related items from
    other inlines, ie. images, to have some relation to other inlines.

    Use as follows::

        class MyInline(LimitedAdminInlineMixin, admin.TabularInline):
            def get_filters(self, request, obj):
                return (('<field_name>', dict(<filters>)),)

    Here, <field_name> is the name of the FK field to be filtered and <filters>
    is a dict of parameters as you would normally specify them in the filter()
    method of QuerySets.

    Modified from:
    https://stackoverflow.com/a/5008984
    https://gist.github.com/dokterbob/828117
    """

    @staticmethod
    def limit_inline_choices(formset, field, empty=False, **filters):
        """
        This function fetches the queryset with available choices for a given
        `field` and filters it based on the criteria specified in filters,
        unless `empty=True`. In this case, no choices will be made available.
        """
        assert field in formset.form.base_fields
        qs = formset.form.base_fields[field].queryset
        if empty:
            formset.form.base_fields[field].queryset = qs.none()
        else:
            qs = qs.filter(**filters)
            formset.form.base_fields[field].queryset = qs

    def get_formset(self, request, obj=None, **kwargs):
        """
        Make sure we can only select variations that relate to the current
        item.
        """
        formset = super(LimitedAdminInlineMixin, self).get_formset(request,
                                                                   obj,
                                                                   **kwargs)
        inline_filters = self.get_filters(request, obj)
        if inline_filters:
            for (field, filters,) in inline_filters:
                self.limit_inline_choices(formset, field, **filters)
        return formset


# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import ResourceManager
from koalixcrm.djangoUserExtension.factories.factory_user_extension import StandardUserExtensionFactory


class StandardResourceManagerFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ResourceManager

    user = factory.SubFactory(StandardUserExtensionFactory)



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from koalixcrm.djangoUserExtension.user_extension.user_extension import UserExtension
from django.utils.translation import ugettext as _


class ResourceManager(models.Model):
    user = models.ForeignKey(UserExtension,
                             on_delete=models.CASCADE,
                             verbose_name=_("User"))


class ResourceManagerAdminView(admin.ModelAdmin):
    list_display = ('id',
                    'user',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('user',)
        }),
    )



# -*- coding: utf-8 -*-
from rest_framework import viewsets
from rest_framework.authentication import BasicAuthentication, SessionAuthentication
from rest_framework.decorators import authentication_classes
from rest_framework.decorators import permission_classes
from rest_framework.permissions import IsAuthenticated

from koalixcrm.accounting.models import Account, AccountingPeriod, Booking, ProductCategory
from koalixcrm.accounting.rest.account_rest import AccountJSONSerializer
from koalixcrm.accounting.rest.accounting_period_rest import AccountingPeriodJSONSerializer
from koalixcrm.accounting.rest.booking_rest import BookingJSONSerializer
from koalixcrm.accounting.rest.product_categorie_rest import ProductCategoryJSONSerializer


class AccountAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows accounts to be created, viewed and modified.
    """
    queryset = Account.objects.all()
    serializer_class = AccountJSONSerializer
    filter_fields = ('account_type',)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(AccountAsJSON, self).dispatch(*args, **kwargs)


class AccountingPeriodAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows accounting periods to be created, viewed and modified.
    """
    queryset = AccountingPeriod.objects.all()
    serializer_class = AccountingPeriodJSONSerializer

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(AccountingPeriodAsJSON, self).dispatch(*args, **kwargs)


class BookingAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows bookings to be created, viewed and modified.
    """
    queryset = Booking.objects.all()
    serializer_class = BookingJSONSerializer

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(BookingAsJSON, self).dispatch(*args, **kwargs)


class ProductCategoryAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows product categories to be created, viewed and modified.
    """
    queryset = ProductCategory.objects.all()
    serializer_class = ProductCategoryJSONSerializer

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(ProductCategoryAsJSON, self).dispatch(*args, **kwargs)



# -*- coding: utf-8 -*-

from django import forms


class TemplateSetMissingForUserExtension(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class UserExtensionMissing(Exception):

    def __init__(self, value):
        self.value = value
        self.view = "/koalixcrm/crm/reporting/user_extension_missing"

    def __str__(self):
        return repr(self.value)


class TooManyUserExtensionsAvailable(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class UserExtensionPhoneAddressMissing(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class UserExtensionEmailAddressMissing(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


# -*- coding: utf-8 -*-
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.accounting.models import Account


class ProductCategory(models.Model):
    title = models.CharField(verbose_name=_("Product Category Title"),
                             max_length=50)
    profit_account = models.ForeignKey(Account,
                                       on_delete=models.CASCADE,
                                       verbose_name=_("Profit Account"),
                                       limit_choices_to={"account_type": "E"},
                                       related_name="db_profit_account")
    loss_account = models.ForeignKey(Account,
                                     on_delete=models.CASCADE,
                                     verbose_name=_("Loss Account"),
                                     limit_choices_to={"account_type": "S"},
                                     related_name="db_loss_account")

    class Meta:
        app_label = "accounting"
        verbose_name = _('Product Category')
        verbose_name_plural = _('Product Categories')

    def __str__(self):
        return self.title


class OptionProductCategory(admin.ModelAdmin):
    list_display = ('title',
                    'profit_account',
                    'loss_account')
    list_display_links = ('title',
                          'profit_account',
                          'loss_account')
    fieldsets = (
        (_('Basics'), {
            'fields': ('title',
                       'profit_account',
                       'loss_account')
        }),
    )
    save_as = True



from rest_framework import serializers
from koalixcrm.crm.reporting.agreement_type import AgreementType


class OptionAgreementTypeJSONSerializer(serializers.HyperlinkedModelSerializer):
    title = serializers.CharField(source='title', read_only=True)
    description = serializers.CharField(source='description', read_only=True)

    class Meta:
        model = AgreementType
        fields = ('title',
                  'description')


class AgreementTypeJSONSerializer(serializers.HyperlinkedModelSerializer):
    title = serializers.CharField(source='title')
    description = serializers.CharField(source='description')

    class Meta:
        model = AgreementType
        fields = ('title',
                  'description')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-05-07 19:57
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0025_auto_20180413_1937'),
    ]

    operations = [
        migrations.RenameField(
            model_name='contact',
            old_name='dateofcreation',
            new_name='date_of_creation',
        ),
        migrations.RenameField(
            model_name='contact',
            old_name='lastmodification',
            new_name='last_modification',
        ),
        migrations.RenameField(
            model_name='contact',
            old_name='lastmodifiedby',
            new_name='last_modified_by',
        ),
        migrations.RenameField(
            model_name='customer',
            old_name='defaultCustomerBillingCycle',
            new_name='default_customer_billing_cycle',
        ),
        migrations.RenameField(
            model_name='customer',
            old_name='ismemberof',
            new_name='is_member_of',
        ),
    ]



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import ProjectLinkType


class RelatedToProjectLinkTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProjectLinkType

    title = "Is related to"
    description = "This project is related with ...."


class RequiresProjectLinkTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProjectLinkType

    title = "This project requires"
    description = "This project requires the completion or the existence of ...."



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.crm.contact.contact import Contact
from koalixcrm.crm.contact.contact import ContactPostalAddress
from koalixcrm.crm.contact.contact import ContactPhoneAddress
from koalixcrm.crm.contact.contact import ContactEmailAddress


class Supplier(Contact):
    offers_shipment_to_customers = models.BooleanField(verbose_name=_("Offers Shipment to Customer"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Supplier')
        verbose_name_plural = _('Suppliers')

    def __str__(self):
        return str(self.id) + ' ' + self.name


class OptionSupplier(admin.ModelAdmin):
    list_display = ('id',
                    'name',
                    'offers_shipment_to_customers')
    fieldsets = (('',
                  {'fields': ('name',
                              'offers_shipment_to_customers')}),)
    inlines = [ContactPostalAddress, ContactPhoneAddress, ContactEmailAddress]
    allow_add = True

    def save_model(self, request, obj, form, change):
        if change:
            obj.last_modified_by = request.user
        else:
            obj.last_modified_by = request.user
            obj.staff = request.user
        obj.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-11-10 17:32
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import filebrowser.fields


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0002_auto_20171009_1949'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='subscription',
            options={'verbose_name': 'Subscription', 'verbose_name_plural': 'Subscriptions'},
        ),
        migrations.AlterModelOptions(
            name='subscriptionevent',
            options={'verbose_name': 'Subscription Event', 'verbose_name_plural': 'Subscription Events'},
        ),
        migrations.AlterModelOptions(
            name='subscriptiontype',
            options={'verbose_name': 'Subscription Type', 'verbose_name_plural': 'Subscription Types'},
        ),
        migrations.AlterField(
            model_name='subscription',
            name='contract',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Contract', verbose_name='Subscription Type'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='subscriptiontype',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='subscriptions.SubscriptionType', verbose_name='Subscription Type'),
        ),
        migrations.AlterField(
            model_name='subscriptionevent',
            name='event',
            field=models.CharField(choices=[('O', 'Offered'), ('C', 'Canceled'), ('S', 'Signed')], max_length=1, verbose_name='Event'),
        ),
        migrations.AlterField(
            model_name='subscriptionevent',
            name='eventdate',
            field=models.DateField(blank=True, null=True, verbose_name='Event Date'),
        ),
        migrations.AlterField(
            model_name='subscriptionevent',
            name='subscriptions',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subscriptions.Subscription', verbose_name='Subscription'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='automaticContractExtension',
            field=models.IntegerField(blank=True, null=True, verbose_name='Automatic Contract Extension (months)'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='automaticContractExtensionReminder',
            field=models.IntegerField(blank=True, null=True, verbose_name='Automatic Contract Extensoin Reminder (days)'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='cancelationPeriod',
            field=models.IntegerField(blank=True, null=True, verbose_name='Cancelation Period (months)'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='contractDocument',
            field=filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True, verbose_name='Contract Documents'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='minimumDuration',
            field=models.IntegerField(blank=True, null=True, verbose_name='Minimum Contract Duration'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='paymentIntervall',
            field=models.IntegerField(blank=True, null=True, verbose_name='Payment Intervall (days)'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-12 19:25
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('djangoUserExtension', '0005_auto_20180612_1924'),
        ('crm', '0031_auto_20180612_1924'),
    ]

    operations = [
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('project_name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Project name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('date_of_creation', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('last_modification', models.DateTimeField(auto_now=True, verbose_name='Last modified')),
                ('default_template_set', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.TemplateSet', verbose_name='Default Template Set')),
                ('last_modified_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_project_last_modified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by')),
                ('project_manager', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_rel_project_staff', to=settings.AUTH_USER_MODEL, verbose_name='Staff')),
            ],
            options={
                'verbose_name': 'Project',
                'verbose_name_plural': 'Projects',
            },
        ),
        migrations.CreateModel(
            name='ProjectLinkType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=300, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
            ],
            options={
                'verbose_name': 'Project Link Type',
                'verbose_name_plural': 'Project Link Type',
            },
        ),
        migrations.CreateModel(
            name='ProjectStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=250, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('is_done', models.BooleanField(verbose_name='Status represents project is done')),
            ],
            options={
                'verbose_name': 'Task Status',
                'verbose_name_plural': 'Task Status',
            },
        ),
        migrations.AddField(
            model_name='project',
            name='project_status',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ProjectStatus', verbose_name='Project Status'),
        ),
    ]



# -*- coding: utf-8 -*-
from rest_framework import serializers
from koalixcrm.crm.reporting.resource_price import ResourcePrice
from koalixcrm.crm.product.currency import Currency
from koalixcrm.crm.product.unit import Unit
from koalixcrm.crm.contact.customer_group import CustomerGroup
from koalixcrm.crm.rest.customer_group_rest import OptionCustomerGroupJSONSerializer
from koalixcrm.crm.rest.currency_rest import CurrencyJSONSerializer
from koalixcrm.crm.rest.unit_rest import OptionUnitJSONSerializer


class OptionResourcePriceJSONSerializer(serializers.HyperlinkedModelSerializer):
    price = serializers.DecimalField(source='price', decimal_places=2, max_digits=5)
    unit = OptionUnitJSONSerializer(source='unit')
    customerGroup = OptionCustomerGroupJSONSerializer(source='customer_group')
    currency = CurrencyJSONSerializer(source='currency', allow_null=False)
    validFrom = serializers.DateField(source='valid_from', allow_null=False)
    validUntil = serializers.DateField(source='valid_until')

    class Meta:
        model = ResourcePrice
        fields = ('price',
                  'currency',
                  'unit',
                  'valid_from',
                  'valid_until',
                  'customer_group')


class ResourcePricesSONSerializer(serializers.HyperlinkedModelSerializer):
    price = serializers.DecimalField(source='price', decimal_places=2, max_digits=5)
    currency = CurrencyJSONSerializer(source='currency', allow_null=False)
    unit = OptionUnitJSONSerializer(source='unit')
    validFrom = serializers.DateField(source='valid_from', allow_null=False)
    validUntil = serializers.DateField(source='valid_until')
    customerGroup = OptionCustomerGroupJSONSerializer(source='customer_group')

    class Meta:
        model = ResourcePrice
        fields = ('price',
                  'currency',
                  'unit',
                  'valid_from',
                  'valid_until',
                  'customer_group')

    def create(self, validated_data):
        resource_price = ResourcePrice()
        # Deserialize currency
        currency = validated_data.pop('currency')
        if currency:
            if currency.get('id', None):
                resource_price.currency = Currency.objects.get(id=currency.get('id', None))
            else:
                resource_price.currency = None
        # Deserialize unit
        unit = validated_data.pop('unit')
        if unit:
            if unit.get('id', None):
                resource_price.unit = Unit.objects.get(id=unit.get('id', None))
            else:
                resource_price.unit = None
        resource_price.save()
        # Deserialize customer group
        customer_group = validated_data.pop('customer_group')
        if customer_group:
            if customer_group.get('id', None):
                resource_price.customer_group = CustomerGroup.objects.get(id=customer_group.get('id', None))
            else:
                resource_price.customer_group = None
        resource_price.save()
        return resource_price

    def update(self, resource_price, validated_data):
        # Deserialize currency
        currency = validated_data.pop('currency')
        if currency:
            if currency.get('id', resource_price.currency):
                resource_price.currency = Currency.objects.get(id=currency.get('id', None))
            else:
                resource_price.currency = resource_price.currency_id
        else:
            resource_price.currency = None
        # Deserialize unit
        unit = validated_data.pop('unit')
        if unit:
            if unit.get('id', resource_price.status):
                resource_price.unit = Unit.objects.get(id=unit.get('id', None))
            else:
                resource_price.unit = resource_price.unit_id
        else:
            resource_price.unit = None
        resource_price.save()
        # Deserialize customer group
        customer_group = validated_data.pop('customer_group')
        if customer_group:
            if customer_group.get('id', resource_price.customer_group):
                resource_price.customer_group = CustomerGroup.objects.get(id=customer_group.get('id', None))
            else:
                resource_price.customer_group = resource_price.customer_group_id
        else:
            resource_price.customer_group = None
        resource_price.save()
        return resource_price






# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import Tax


class StandardTaxFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Tax
        django_get_or_create = ('name',)

    tax_rate = "7.7"
    name = "Swiss MwSt 7.7%"
    account_activa = None
    account_passiva = None



# -*- coding: utf-8 -*-

from datetime import *
from django.db import models
from django.utils.translation import ugettext as _
from filebrowser.fields import FileBrowseField
from koalixcrm.subscriptions.const.events import *
import koalixcrm.crm.documents


class Subscription(models.Model):
    contract = models.ForeignKey('crm.Contract', on_delete=models.CASCADE, verbose_name=_('Subscription Type'))
    subscription_type = models.ForeignKey('SubscriptionType', on_delete=models.CASCADE, verbose_name=_('Subscription Type'), null=True)

    def create_subscription_from_contract(self, contract):
        self.contract = contract
        self.save()
        return self

    def create_quote(self):
        quote = koalixcrm.crm.documents.quote.Quote()
        quote.contract = self.contract
        quote.discount = 0
        quote.staff = self.contract.staff
        quote.customer = self.contract.defaultcustomer
        quote.status = 'C'
        quote.currency = self.contract.defaultcurrency
        quote.valid_until = date.today().__str__()
        quote.date_of_creation = date.today().__str__()
        quote.save()
        return quote

    def create_invoice(self):
        invoice = koalixcrm.crm.documents.invoice.Invoice()
        invoice.contract = self.contract
        invoice.discount = 0
        invoice.staff = self.contract.staff
        invoice.customer = self.contract.default_customer
        invoice.status = 'C'
        invoice.currency = self.contract.default_currency
        invoice.payable_until = date.today() + timedelta(
            days=self.contract.defaultcustomer.defaultCustomerBillingCycle.timeToPaymentDate)
        invoice.date_of_creation = date.today().__str__()
        invoice.save()
        return invoice

    class Meta:
        app_label = "subscriptions"
        verbose_name = _('Subscription')
        verbose_name_plural = _('Subscriptions')


class SubscriptionEvent(models.Model):
    subscriptions = models.ForeignKey('Subscription',
                                      on_delete=models.CASCADE,
                                      verbose_name=_('Subscription'))
    event_date = models.DateField(verbose_name=_("Event Date"),
                                  blank=True, null=True)
    event = models.CharField(max_length=1, choices=SUBSCRITIONEVENTS,
                             verbose_name=_('Event'))

    def __str__(self):
        return self.event

    class Meta:
        app_label = "subscriptions"
        verbose_name = _('Subscription Event')
        verbose_name_plural = _('Subscription Events')


class SubscriptionType(models.Model):
    product_type = models.ForeignKey('crm.ProductType',
                                     verbose_name=_('Product Type'),
                                     on_delete=models.deletion.SET_NULL,
                                     null=True,
                                     blank=True)
    cancellation_period = models.IntegerField(verbose_name=_("Cancellation Period (months)"),
                                              blank=True,
                                              null=True)
    automatic_contract_extension = models.IntegerField(verbose_name=_("Automatic Contract Extension (months)"),
                                                       blank=True,
                                                       null=True)
    automatic_contract_extension_reminder = models.IntegerField(
        verbose_name=_("Automatic Contract Extension Reminder (days)"),
        blank=True,
        null=True)
    minimum_duration = models.IntegerField(verbose_name=_("Minimum Contract Duration"),
                                           blank=True,
                                           null=True)
    payment_interval = models.IntegerField(verbose_name=_("Payment Interval (days)"),
                                           blank=True,
                                           null=True)
    contract_document = FileBrowseField(verbose_name=_("Contract Documents"),
                                        blank=True,
                                        null=True,
                                        max_length=200)

    class Meta:
        app_label = "subscriptions"
        verbose_name = _('Subscription Type')
        verbose_name_plural = _('Subscription Types')



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _
from django.contrib import admin


class ProjectLinkType(models.Model):
    title = models.CharField(verbose_name=_("Title"), max_length=300, blank=False, null=False)
    description = models.TextField(verbose_name=_("Text"), blank=True, null=True)

    class Meta:
        app_label = "crm"
        verbose_name = _('Project Link Type')
        verbose_name_plural = _('Project Link Type')

    def __str__(self):
        return str(self.id) + " " + str(self.title)


class OptionProjectLinkType(admin.ModelAdmin):
    list_display = ('id',
                    'title',
                    'description')

    fieldsets = (
        (_('ProjectLinkType'), {
            'fields': ('title',
                       'description')
        }),
    )
    save_as = True



# -*- coding: utf-8 -*-

from koalixcrm.crm.models import PurchaseConfirmation
from koalixcrm.crm.factories.factory_sales_document import StandardSalesDocumentFactory


class StandardPurchaseConfirmationFactory(StandardSalesDocumentFactory):
    class Meta:
        model = PurchaseConfirmation





# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.crm.reporting.resource_price import ResourcePrice
from decimal import *
from django.core.exceptions import ValidationError
from django.forms import BaseInlineFormSet
from koalixcrm.crm.exceptions import ReportingPeriodNotFound


class Estimation(models.Model):
    """The estimation describes the estimated amount of resources which is still required to finish a task
    the estimation is done within a reporting period that is not yet closed. The estimation is done only considering
    all effective efforts that was reported in the previous and closed reporting periods"""

    task = models.ForeignKey("Task",
                             on_delete=models.CASCADE,
                             verbose_name=_('Task'),
                             blank=False,
                             null=False)
    resource = models.ForeignKey("Resource",
                                 on_delete=models.CASCADE)
    date_from = models.DateField(verbose_name=_("Estimation From"),
                                 blank=False,
                                 null=False)
    date_until = models.DateField(verbose_name=_("Estimation Until"),
                                  blank=False,
                                  null=False)
    amount = models.DecimalField(verbose_name=_("Amount"),
                                 max_digits=5,
                                 decimal_places=2,
                                 blank=True,
                                 null=True)
    status = models.ForeignKey("EstimationStatus",
                               on_delete=models.CASCADE,
                               verbose_name=_('Status of the estimation'),
                               blank=False,
                               null=False)
    reporting_period = models.ForeignKey("ReportingPeriod",
                                         on_delete=models.CASCADE,
                                         verbose_name=_('Reporting Period based on which the estimation was done'),
                                         blank=False,
                                         null=False)

    def duration_in_days(self):
        """The function returns the calculated difference between the date_until and the date_from and returns the value
        as number of days

        Args:
        no arguments needed

        Returns:
        difference between date_until and date_from (Integer)

        Raises:
        No exceptions planned"""
        duration_time_delta = self.date_until - self.date_from
        duration = duration_time_delta.days
        return duration

    def calculated_costs(self, bucket_start=None, bucket_end=None):
        """The function returns the calculated costs in total or the calculated costs within a specific start and
        stop-frame.

        Args:
        start (datetime.date)
        stop (datetime.date)

        Returns:
        planned costs (Decimal), 0 if no price can be found

        Raises:
        No exceptions planned"""
        default_resource_price = ResourcePrice.objects.filter(resource=self.resource)
        overall_costs = 0
        estimation_lies_outside_bucket = (bucket_end <= self.date_from) or bucket_start >= self.date_until
        if bucket_start >= self.date_from:
            later_start_date = bucket_start
        else:
            later_start_date = self.date_from
        if bucket_end <= self.date_until:
            earlier_end_date = bucket_end
        else:
            earlier_end_date = self.date_until

        if estimation_lies_outside_bucket:
            selected_duration = 0
        else:
            selected_duration_time_delta = earlier_end_date - later_start_date
            selected_duration = selected_duration_time_delta.days
            # Add one day because the buckets may not overlap each other. By this fact we always loose one day
            selected_duration += 1

        if len(default_resource_price) == 0:
            overall_costs = 0
        else:
            for resource_price in default_resource_price:
                overall_costs = self.amount*resource_price.price
                break
        if self.duration_in_days() <= selected_duration:
            costs = overall_costs
        else:
            costs = overall_costs * Decimal((selected_duration / self.duration_in_days()))
        return costs

    def __str__(self):
        return _("Estimation of Resource Consumption") + ": " + str(self.id)

    class Meta:
        app_label = "crm"
        verbose_name = _('Estimation of Resource Consumption')
        verbose_name_plural = _('Estimation of Resource Consumptions')


class EstimationAdminForm(BaseInlineFormSet):
    def clean(self):
        """Check that the estimation is only attached to a reporting period which is not yet closed,
        also check that the date_from is at least one day before the date_until"""
        for f in self.forms:
            if any(f.errors):
                pass
            else:
                if 'date_from' in f.cleaned_data:
                    date_from = f.cleaned_data['date_from']
                else:
                    break
                date_until = f.cleaned_data['date_until']
                reporting_period = f.cleaned_data['reporting_period']
                task = f.cleaned_data['task']
                if f.cleaned_data['id']:
                    limit_of_acceptable_estimations = 1
                else:
                    limit_of_acceptable_estimations = 0
                existing_estimations = Estimation.objects.filter(reporting_period=reporting_period, task=task)
                if len(existing_estimations) > limit_of_acceptable_estimations:
                    raise ValidationError('There may only be one estimation per reporting period per task')
                try:
                    predecessor_reporting_period = reporting_period.get_predecessor(reporting_period,
                                                                                    reporting_period.project)
                    if not predecessor_reporting_period.status.is_done:
                        raise ValidationError('Please select a reporting period which has a predecessor'
                                              ' reporting period which is already in state "done"')
                except ReportingPeriodNotFound:
                    pass
                if reporting_period.status.is_done:
                    raise ValidationError('Please select a reporting period which is not yet in state "done"')
                if date_from >= date_until:
                    raise ValidationError('The date until must be at least one day after date from')


class EstimationInlineAdminView(admin.TabularInline):
    model = Estimation
    formset = EstimationAdminForm
    fieldsets = (
        (_('Work'), {
            'fields': ('task',
                       'amount',
                       'resource',
                       'date_from',
                       'date_until',
                       'status',
                       'reporting_period')
        }),
    )
    extra = 1



# -*- coding: utf-8 -*-
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from django import forms

from koalixcrm.accounting.const.accountTypeChoices import *
from koalixcrm.accounting.exceptions import AccountingPeriodNotFound
from koalixcrm.crm.documents.pdf_export import PDFExport


class Account(models.Model):
    account_number = models.IntegerField(verbose_name=_("Account Number"))
    title = models.CharField(verbose_name=_("Account Title"),
                             max_length=50)
    account_type = models.CharField(verbose_name=_("Account Type"),
                                    max_length=1,
                                    choices=ACCOUNTTYPECHOICES)
    description = models.TextField(verbose_name=_("Description"), null=True, blank=True)
    is_open_reliabilities_account = models.BooleanField(verbose_name=_("Is The Open Liabilities Account"))
    is_open_interest_account = models.BooleanField(verbose_name=_("Is The Open Interests Account"))
    is_product_inventory_activa = models.BooleanField(verbose_name=_("Is a Product Inventory Account"))
    is_a_customer_payment_account = models.BooleanField(verbose_name=_("Is a Customer Payment Account"))

    def sum_of_all_bookings(self):
        calculated_sum = self.all_bookings(from_account=False) - self.all_bookings(from_account=True)
        if self.account_type == 'E' or self.account_type == 'L':
            calculated_sum = 0 - calculated_sum
        return calculated_sum

    sum_of_all_bookings.short_description = _("Value")

    def sum_of_all_bookings_within_accounting_period(self, accounting_period):
        calculated_sum = self.all_bookings_within_accounting_period(from_account=False,
                                                                    accounting_period=accounting_period) - \
                         self.all_bookings_within_accounting_period(from_account=True,
                                                                    accounting_period=accounting_period)
        if self.account_type == 'E' or self.account_type == 'L':
            calculated_sum = -calculated_sum
        return calculated_sum

    def sum_of_all_bookings_before_accounting_period(self, current_accounting_period):
        try:
            accounting_periods = current_accounting_period.get_all_prior_accounting_periods()
        except AccountingPeriodNotFound:
            return 0
        sum_of_all_bookings = 0
        for accounting_period in accounting_periods:
            sum_of_all_bookings += self.all_bookings_within_accounting_period(from_account=False,
                                                              accounting_period=accounting_period) - self.all_bookings_within_accounting_period(
                from_account=True, accounting_period=accounting_period)
        if self.account_type == 'E' or self.account_type == 'L':
            sum_of_all_bookings = -sum_of_all_bookings
        return sum_of_all_bookings

    def sum_of_all_bookings_through_now(self, current_accounting_period):
        within_accounting_period = self.sum_of_all_bookings_within_accounting_period(current_accounting_period)
        before_accounting_period = self.sum_of_all_bookings_before_accounting_period(current_accounting_period)
        current_value = within_accounting_period + before_accounting_period
        return current_value

    def all_bookings(self, from_account):
        from koalixcrm.accounting.models import Booking
        sum_all = 0
        if from_account:
            bookings = Booking.objects.filter(from_account=self.id)
        else:
            bookings = Booking.objects.filter(to_account=self.id)

        for booking in list(bookings):
            sum_all += booking.amount

        return sum_all

    def all_bookings_within_accounting_period(self, from_account, accounting_period):
        from koalixcrm.accounting.models import Booking
        sum = 0
        if from_account:
            bookings = Booking.objects.filter(from_account=self.id, accounting_period=accounting_period.id)
        else:
            bookings = Booking.objects.filter(to_account=self.id, accounting_period=accounting_period.id)

        for booking in list(bookings):
            sum += booking.amount

        return sum

    def serialize_to_xml(self, accounting_period):
        objects = [self, ]
        main_xml = PDFExport.write_xml(objects)
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='accounting.account']",
                                                       "sum_of_all_bookings_within_accounting_period",
                                                       self.sum_of_all_bookings_within_accounting_period(accounting_period))
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='accounting.account']",
                                                       "sum_of_all_bookings_through_now",
                                                       self.sum_of_all_bookings_through_now(accounting_period))
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='accounting.account']",
                                                       "sum_of_all_bookings_before_accounting_period",
                                                       self.sum_of_all_bookings_before_accounting_period(accounting_period))
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='accounting.account']",
                                                       "sum_of_all_bookings_through_now",
                                                       self.sum_of_all_bookings())
        return main_xml

    def __str__(self):
        return self.account_number.__str__() + " " + self.title

    class Meta:
        app_label = "accounting"
        verbose_name = _('Account')
        verbose_name_plural = _('Account')
        ordering = ['account_number']


class AccountForm(forms.ModelForm):
    """AccountForm is used to overwrite the clean method of the
    original form and to add an additional checks to the model"""

    class Meta:
        model = Account
        fields = '__all__'

    def clean(self):
        super(AccountForm, self).clean()
        errors = []
        if self.cleaned_data['is_open_reliabilities_account']:
            open_reliabilities_account = Account.objects.filter(is_open_reliabilities_account=True)
            if self.cleaned_data['account_type'] != "L":
                errors.append(_('The open liabilities account must be a liabities account'))
            elif open_reliabilities_account:
                errors.append(_('There may only be one open liablities account in the system'))
        if self.cleaned_data['is_open_interest_account']:
            open_interest_account = Account.objects.filter(is_open_interest_account=True)
            if self.cleaned_data['account_type'] != "A":
                errors.append(_('The open interests account must be an asset account'))
            elif open_interest_account:
                errors.append(_('There may only be one open intrests account in the system'))
        if self.cleaned_data['is_a_customer_payment_account']:
            if self.cleaned_data['account_type'] != "A":
                errors.append(_('A customer payment account must be an asset account'))
        if self.cleaned_data['is_product_inventory_activa']:
            if self.cleaned_data['account_type'] != "A":
                errors.append(_('A product inventory account must be an asset account'))
        if len(errors) > 0:
            raise forms.ValidationError(errors)
        return self.cleaned_data


class OptionAccount(admin.ModelAdmin):
    list_display = ('account_number',
                    'account_type',
                    'title',
                    'sum_of_all_bookings')
    list_display_links = ('account_number',
                          'account_type',
                          'title',
                          'sum_of_all_bookings')
    fieldsets = ((_('Basic'),
                  {'fields': ('account_number',
                              'account_type',
                              'title',
                              'description',
                              'is_open_reliabilities_account',
                              'is_open_interest_account',
                              'is_product_inventory_activa',
                              'is_a_customer_payment_account')}),)
    save_as = True

    form = AccountForm



# -*- coding: utf-8 -*-
from os import path
from wsgiref.util import FileWrapper
from django.http import Http404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from koalixcrm.crm.exceptions import *
from koalixcrm.djangoUserExtension.exceptions import *
from django.utils.translation import ugettext as _


def export_pdf(calling_model_admin, request, whereToCreateFrom, whatToCreate, redirectTo):
    """This method exports PDFs provided by different Models in the accounting application

        Args:
          calling_model_admin (ModelAdmin):  The calling ModelAdmin must be provided for error message response.
          request: The request User is required to get the Calling User TemplateSets and to know where to
          save the error message whereToCreateFrom (Model):  The model from which a PDF should be exported
          whatToCreate (str): What document Type that has to be created
          redirectTo (str): String that describes to where the method sould redirect in case of an error

        Returns:
          HTTpResponse with a PDF when successful
          HTTpResponseRedirect when not successful

        Raises:
          Http404 exception if anything goes wrong"""
    try:
        pdf = whereToCreateFrom.createPDF(request.user, whatToCreate)
        response = HttpResponse(FileWrapper(open(pdf, 'rb')), content_type='application/pdf')
        response['Content-Length'] = path.getsize(pdf)
    except (TemplateSetMissing, UserExtensionMissing) as e:
        if e.isinstance(UserExtensionMissing):
            response = HttpResponseRedirect(redirectTo)
            calling_model_admin.message_user(request, _("User Extension Missing"))
        elif e.isinstance(TemplateSetMissing):
            response = HttpResponseRedirect(redirectTo)
            calling_model_admin.message_user(request, _("Templateset Missing"))
        else:
            raise Http404
    return response


def export_xml(callingModelAdmin, request, whereToCreateFrom, whatToCreate, redirectTo):
    """This method exports XMLs provided by different Models in the accounting application

        Args:
          callingModelAdmin (ModelAdmin):  The calling ModelAdmin must be provided for error message response.
          request: The request User is required to get the Calling User TemplateSets and to know where
          to save the error message hereToCreateFrom (Model):  The model from which a PDF should be exported
          whatToCreate (str): What objects that have to be serialized
          redirectTo (str): String that describes to where the method sould redirect in case of an error

        Returns:
          HTTpResponse with a PDF when successful
          HTTpResponseRedirect when not successful

        Raises:
          raises Http404 exception if anything goes wrong"""
    try:
        xml = whereToCreateFrom.createXML(request.user, whatToCreate)
        response = HttpResponse(FileWrapper(open(xml, 'rb')), mimetype='application/xml')
        response['Content-Length'] = path.getsize(xml)
    except (TemplateSetMissing, UserExtensionMissing) as e:
        if e.isinstance(UserExtensionMissing):
            response = HttpResponseRedirect(redirectTo)
            callingModelAdmin.message_user(request, _("User Extension Missing"))
        elif e.isinstance(TemplateSetMissing):
            response = HttpResponseRedirect(redirectTo)
            callingModelAdmin.message_user(request, _("Templateset Missing"))
        else:
            raise Http404
    return response



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-11 19:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0028_auto_20180611_1835'),
    ]

    operations = [
        migrations.AlterField(
            model_name='generictasklink',
            name='task_link_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.TaskLinkType', verbose_name='Task Link Type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-12-10 21:26
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import filebrowser.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0003_auto_20171110_1732'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=100, null=True, verbose_name='Title')),
                ('xsl_file', filebrowser.fields.FileBrowseField(max_length=200, verbose_name='XSL File')),
                ('fop_config_file', filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True, verbose_name='FOP Configuration File')),
                ('logo', filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True, verbose_name='Logo for the PDF generation')),
            ],
            options={
                'verbose_name': 'Document template',
                'verbose_name_plural': 'Document templates',
            },
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='addresser',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='balancesheetXSLFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='bankingaccountref',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='deilveryorderXSLFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='footerTextpurchaseorders',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='footerTextsalesorders',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='fopConfigurationFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='headerTextpurchaseorders',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='headerTextsalesorders',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='invoiceXSLFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='logo',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='organisationname',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='pagefooterleft',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='pagefootermiddle',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='profitLossStatementXSLFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='purchaseconfirmationXSLFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='purchaseorderXSLFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='quoteXSLFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='reminder1XSLFile',
        ),
        migrations.RemoveField(
            model_name='templateset',
            name='reminder2XSLFile',
        ),
        migrations.CreateModel(
            name='BalanceSheetTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Balance sheet template',
                'verbose_name_plural': 'Balance sheet templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.CreateModel(
            name='DeliveryNoteTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Delivery note template',
                'verbose_name_plural': 'Delivery note templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.CreateModel(
            name='InvoiceTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Invoice template',
                'verbose_name_plural': 'Invoice templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.CreateModel(
            name='PaymentReminderTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Payment reminder template',
                'verbose_name_plural': 'Payment reminder templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.CreateModel(
            name='ProfitLossStatementTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Profit loss statement template',
                'verbose_name_plural': 'Profit loss statement templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.CreateModel(
            name='PurchaseConfirmationTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Purchase confirmation template',
                'verbose_name_plural': 'Purchase confirmation templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.CreateModel(
            name='PurchaseOrderTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Purchase order template',
                'verbose_name_plural': 'Purchase order templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.CreateModel(
            name='QuoteTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Quote template',
                'verbose_name_plural': 'Quote templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.DeleteModel(
            name='XSLFile',
        ),
        migrations.AddField(
            model_name='templateset',
            name='balance_sheet_statement_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.BalanceSheetTemplate'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='delivery_note_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.DeliveryNoteTemplate'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='invoice_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.InvoiceTemplate'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='payment_reminder_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.PaymentReminderTemplate'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='profit_loss_statement_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.ProfitLossStatementTemplate'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='purchase_confirmation_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.PurchaseConfirmationTemplate'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='purchase_order_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.PurchaseOrderTemplate'),
        ),
        migrations.AddField(
            model_name='templateset',
            name='quote_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.QuoteTemplate'),
        ),
    ]



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import CustomerBillingCycle


class StandardCustomerBillingCycleFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = CustomerBillingCycle
        django_get_or_create = ('name',)

    name = "This is a test billing cycle"
    time_to_payment_date = 30
    payment_reminder_time_to_payment = 20



# -*- coding: utf-8 -*-

from selenium.common.exceptions import NoSuchElementException
import time
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys


def assert_when_element_does_not_exist(testcase, xpath):
    try:
        testcase.selenium.find_element_by_xpath(xpath)
    except NoSuchElementException:
        testcase.assertTrue(True, xpath+" does not exist")


def assert_when_element_exists(testcase, xpath):
    try:
        testcase.selenium.find_element_by_xpath(xpath)
    except NoSuchElementException:
        testcase.assertTrue(True, xpath+" does exist")


def assert_when_element_is_not_equal_to(testcase, xpath, string):
    try:
        element = testcase.selenium.find_element_by_xpath(xpath)
        if element.text != string:
            print("issue")
    except NoSuchElementException:
        return


def create_sales_document_from_reference(test_case,
                                         timeout,
                                         reference_type,
                                         reference_id,
                                         document_type,
                                         action_name,
                                         template_name,
                                         template_to_select):
    selenium = test_case.selenium
    selenium.get('%s%s' % (test_case.live_server_url, '/admin/crm/'+reference_type+'/'))
    time.sleep(1)
    assert_when_element_does_not_exist(test_case,
                                       '/html/body/div/article/div/form/section/div/table/tbody/tr/td[1]/input')
    assert_when_element_does_not_exist(test_case, '/html/body/div/article/div/form/footer/ul/li/div/select')
    assert_when_element_does_not_exist(test_case, '/html/body/div/article/div/form/footer/ul/li/div/button')
    contract_1 = selenium.find_element_by_xpath(
        '/html/body/div/article/div/form/section/div/table/tbody/tr/td[1]/input')
    if not contract_1.is_selected():
        contract_1.send_keys(Keys.SPACE)
    action_create_offer = selenium.find_element_by_xpath(
        '/html/body/div/article/div/form/footer/ul/li/div/select/option[@value="'+action_name+'"]')
    action_create_offer.click()
    ok_button = selenium.find_element_by_xpath('/html/body/div/article/div/form/footer/ul/li/div/button')
    ok_button.send_keys(Keys.RETURN)
    time.sleep(1)
    try:
        element_present = expected_conditions.presence_of_element_located((By.ID, 'id_title'))
        WebDriverWait(selenium, timeout).until(element_present)
    except TimeoutException:
        print("Timed out waiting for page to load")
    assert_when_element_does_not_exist(test_case, '/html/body/div/article/ul/li')
    document_type_template = selenium.find_element_by_xpath(
        '//*[@id="id_'+template_name+'"]/option[@value="' +
        template_to_select.id.__str__() + '"]')
    document_type_template.click()
    save_button = selenium.find_element_by_xpath('/html/body/div/article/div/form/div/footer/ul/li[2]/input')
    save_button.send_keys(Keys.RETURN)
    time.sleep(1)
    selenium.get('%s%s' % (test_case.live_server_url, '/admin/crm/'+reference_type+'/'))
    contract_1 = selenium.find_element_by_xpath(
        '/html/body/div/article/div/form/section/div/table/tbody/tr/td[1]/input')
    if not contract_1.is_selected():
        contract_1.send_keys(Keys.SPACE)
    action_create_sales_document = selenium.find_element_by_xpath(
        '/html/body/div/article/div/form/footer/ul/li/div/select/option[@value="'+action_name+'"]')
    action_create_sales_document.click()
    ok_button = selenium.find_element_by_xpath('/html/body/div/article/div/form/footer/ul/li/div/button')
    ok_button.send_keys(Keys.RETURN)
    time.sleep(1)
    try:
        element_present = expected_conditions.presence_of_element_located((By.ID, 'id_title'))
        WebDriverWait(selenium, timeout).until(element_present)
    except TimeoutException:
        print("Timed out waiting for page to load")
    sales_documents = document_type.objects.filter(contract=test_case.test_contract)
    for sales_document in sales_documents:
        if sales_document.id != reference_id:
            test_case.assertEqual(type(sales_document), document_type)



# -*- coding: utf-8 -*-

from django.contrib import admin
from koalixcrm.crm.documents.quote import Quote, OptionQuote
from koalixcrm.crm.documents.purchase_confirmation import PurchaseConfirmation, OptionPurchaseConfirmation
from koalixcrm.crm.documents.delivery_note import DeliveryNote, OptionDeliveryNote
from koalixcrm.crm.documents.invoice import Invoice, OptionInvoice
from koalixcrm.crm.documents.payment_reminder import PaymentReminder, OptionPaymentReminder
from koalixcrm.crm.documents.purchase_order import PurchaseOrder, OptionPurchaseOrder
from koalixcrm.crm.documents.contract import Contract, OptionContract
from koalixcrm.crm.product.tax import Tax, OptionTax
from koalixcrm.crm.product.unit import Unit, OptionUnit
from koalixcrm.crm.product.product_type import ProductType, ProductTypeAdminView
from koalixcrm.crm.product.currency import Currency, OptionCurrency
from koalixcrm.crm.contact.customer import Customer, OptionCustomer
from koalixcrm.crm.contact.supplier import Supplier, OptionSupplier
from koalixcrm.crm.contact.customer_group import CustomerGroup, OptionCustomerGroup
from koalixcrm.crm.contact.customer_billing_cycle import CustomerBillingCycle, OptionCustomerBillingCycle
from koalixcrm.crm.contact.person import Person
from koalixcrm.crm.contact.contact import OptionPerson, CallForContact, VisitForContact
from koalixcrm.crm.contact.call import OptionCall, OptionVisit
from koalixcrm.crm.reporting.task import Task, TaskAdminView
from koalixcrm.crm.reporting.task_link_type import TaskLinkType, OptionTaskLinkType
from koalixcrm.crm.reporting.task_status import TaskStatus, OptionTaskStatus
from koalixcrm.crm.reporting.estimation_status import EstimationStatus, EstimationStatusAdminView
from koalixcrm.crm.reporting.agreement_status import AgreementStatus, AgreementStatusAdminView
from koalixcrm.crm.reporting.agreement_type import AgreementType, AgreementTypeAdminView
from koalixcrm.crm.reporting.resource_type import ResourceType, ResourceTypeAdminView
from koalixcrm.crm.reporting.human_resource import HumanResource, HumanResourceAdminView
from koalixcrm.crm.reporting.resource_manager import ResourceManager, ResourceManagerAdminView
from koalixcrm.crm.reporting.project import Project, ProjectAdminView
from koalixcrm.crm.reporting.project_link_type import ProjectLinkType, OptionProjectLinkType
from koalixcrm.crm.reporting.project_status import ProjectStatus, OptionProjectStatus
from koalixcrm.crm.reporting.work import Work, WorkAdminView
from koalixcrm.crm.reporting.reporting_period import ReportingPeriod, ReportingPeriodAdmin
from koalixcrm.crm.reporting.reporting_period_status import ReportingPeriodStatus, OptionReportingPeriodStatus


admin.site.register(Customer, OptionCustomer)
admin.site.register(CustomerGroup, OptionCustomerGroup)
admin.site.register(CustomerBillingCycle, OptionCustomerBillingCycle)
admin.site.register(Supplier, OptionSupplier)
admin.site.register(Person, OptionPerson)
admin.site.register(CallForContact, OptionCall)
admin.site.register(VisitForContact, OptionVisit)

admin.site.register(Contract, OptionContract)
admin.site.register(Quote, OptionQuote)
admin.site.register(PurchaseConfirmation, OptionPurchaseConfirmation)
admin.site.register(DeliveryNote, OptionDeliveryNote)
admin.site.register(Invoice, OptionInvoice)
admin.site.register(PaymentReminder, OptionPaymentReminder)
admin.site.register(PurchaseOrder, OptionPurchaseOrder)

admin.site.register(Unit, OptionUnit)
admin.site.register(Currency, OptionCurrency)
admin.site.register(Tax, OptionTax)
admin.site.register(ProductType, ProductTypeAdminView)

admin.site.register(Task, TaskAdminView)
admin.site.register(TaskLinkType, OptionTaskLinkType)
admin.site.register(TaskStatus, OptionTaskStatus)
admin.site.register(EstimationStatus, EstimationStatusAdminView)
admin.site.register(AgreementStatus, AgreementStatusAdminView)
admin.site.register(AgreementType, AgreementTypeAdminView)
admin.site.register(Work, WorkAdminView)
admin.site.register(HumanResource, HumanResourceAdminView)
admin.site.register(ResourceType, ResourceTypeAdminView)
admin.site.register(ResourceManager, ResourceManagerAdminView)
admin.site.register(Project, ProjectAdminView)
admin.site.register(ProjectLinkType, OptionProjectLinkType)
admin.site.register(ProjectStatus, OptionProjectStatus)
admin.site.register(ReportingPeriod, ReportingPeriodAdmin)
admin.site.register(ReportingPeriodStatus, OptionReportingPeriodStatus)



# -*- coding: utf-8 -*-

from datetime import *
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from django import forms
from koalixcrm.accounting.models import Account
from koalixcrm.crm.documents.pdf_export import PDFExport
from koalixcrm.accounting.exceptions import AccountingPeriodNotFound
from koalixcrm.accounting.exceptions import TemplateSetMissingInAccountingPeriod
from koalixcrm.accounting.models import InlineBookings


class AccountingPeriod(models.Model):
    """Accounting period represents the equivalent of the business logic element of a fiscal year
    the accounting period is referred in the booking and is used as a supporting object to generate
    balance sheets and profit/loss statements"""
    title = models.CharField(max_length=200, verbose_name=_("Title"))  # For example "Year 2009", "1st Quarter 2009"
    begin = models.DateField(verbose_name=_("Begin"))
    end = models.DateField(verbose_name=_("End"))
    template_set_balance_sheet = models.ForeignKey("djangoUserExtension.DocumentTemplate",
                                                   on_delete=models.CASCADE,
                                                   verbose_name=_("Referred template for balance sheet"),
                                                   related_name='db_balancesheet_template_set',
                                                   null=True,
                                                   blank=True)
    template_profit_loss_statement = models.ForeignKey("djangoUserExtension.DocumentTemplate",
                                                       on_delete=models.CASCADE,
                                                       verbose_name=_("Referred template for profit, loss statement"),
                                                       related_name='db_profit_loss_statement_template_set',
                                                       null=True,
                                                       blank=True)

    def get_template_set(self, template_set):
        if template_set == self.template_set_balance_sheet:
            if self.template_set_balance_sheet:
                return self.template_set_balance_sheet
            else:
                raise TemplateSetMissingInAccountingPeriod((_("Template Set for balance sheet " +
                                                              "is missing in Accounting Period" + str(self))))
        elif template_set == self.template_profit_loss_statement:
            if self.template_profit_loss_statement:
                return self.template_profit_loss_statement
            else:
                raise TemplateSetMissingInAccountingPeriod((_("Template Set for profit loss statement" +
                                                              " is missing in Accounting Period" + str(self))))

    def get_fop_config_file(self, template_set):
        template_set = self.get_template_set(template_set)
        return template_set.get_fop_config_file()

    def get_xsl_file(self, template_set):
        template_set = self.get_template_set(template_set)
        return template_set.get_xsl_file()

    def create_pdf(self, template_set, printed_by):
        import koalixcrm.crm
        return koalixcrm.crm.documents.pdfexport.PDFExport.create_pdf(self, template_set, printed_by)

    def overall_earnings(self):
        earnings = 0
        accounts = Account.objects.all()
        for account in list(accounts):
            if account.account_type == "E":
                earnings += account.sum_of_all_bookings_within_accounting_period(self)
        return earnings

    def overall_spendings(self):
        spendings = 0
        accounts = Account.objects.all()
        for account in list(accounts):
            if account.account_type == "S":
                spendings += account.sum_of_all_bookings_within_accounting_period(self)
        return spendings

    def overall_assets(self):
        assets = 0
        accounts = Account.objects.all()
        for account in list(accounts):
            if account.account_type == "A":
                assets += account.sum_of_all_bookings_through_now(self)
        return assets

    def overall_liabilities(self):
        liabilities = 0
        accounts = Account.objects.all()
        for account in list(accounts):
            if account.account_type == "L":
                liabilities += account.sum_of_all_bookings_through_now(self)
        return liabilities

    def serialize_to_xml(self):
        objects = [self, ]
        main_xml = PDFExport.write_xml(objects)
        accounts = Account.objects.all()
        for account in accounts:
            account_xml = account.serialize_to_xml(self)
            main_xml = PDFExport.merge_xml(main_xml, account_xml)
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='accounting.accountingperiod']",
                                                       "Overall_Earnings",
                                                       self.overall_earnings())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='accounting.accountingperiod']",
                                                       "Overall_Spendings",
                                                       self.overall_spendings())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='accounting.accountingperiod']",
                                                       "Overall_Assets",
                                                       self.overall_assets())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='accounting.accountingperiod']",
                                                       "Overall_Liabilities",
                                                       self.overall_liabilities())
        return main_xml

    @staticmethod
    def get_current_valid_accounting_period():
        """Returns the accounting period that is currently valid. Valid is an accounting_period when the current date
          lies between begin and end of the accounting_period

        Args:
          no arguments

        Returns:
          accounting_period (AccoutingPeriod)

        Raises:
          AccountingPeriodNotFound when there is no valid accounting Period"""
        current_valid_accounting_period = None
        for accounting_period in AccountingPeriod.objects.all():
            if accounting_period.begin < date.today() and accounting_period.end > date.today():
                return accounting_period
        if not current_valid_accounting_period:
            raise AccountingPeriodNotFound("The accounting period was not found")

    def get_all_prior_accounting_periods(self):
        """Returns the accounting period that is currently valid. Valid is an accountingPeriod when the current date
          lies between begin and end of the accountingPeriod

        Args:
          no arguments

        Returns:
          accounting_period (List of AccoutingPeriod)

        Raises:
          AccountingPeriodNotFound when there is no valid accounting Period"""
        accounting_periods = []
        for accounting_period in AccountingPeriod.objects.all():
            if accounting_period.end < self.begin:
                accounting_periods.append(accounting_period)
        if accounting_periods == []:
            raise AccountingPeriodNotFound("Accounting Period does not exist")
        return accounting_periods

    def __str__(self):
        return self.title

        # TODO: def createNewAccountingPeriod() Neues Geschäftsjahr erstellen

    class Meta:
        app_label = "accounting"
        verbose_name = _('Accounting Period')
        verbose_name_plural = _('Accounting Periods')


class AccountingPeriodForm(forms.ModelForm):
    """AccountingPeriodForm is used to overwrite the clean method of the
    original form and to add an additional check to the model"""

    class Meta:
        model = AccountingPeriod
        fields = '__all__'

    def clean(self):
        super(AccountingPeriodForm, self).clean()
        errors = []
        try:
            if self.cleaned_data['begin'] > self.cleaned_data['end']:
                errors.append(_('The begin date cannot be later than the end date.'))
        except KeyError:
            errors.append(_('The begin and the end date may not be empty'))
        if errors:
            raise forms.ValidationError(errors)
        return self.cleaned_data


class OptionAccountingPeriod(admin.ModelAdmin):
    list_display = ('title',
                    'begin',
                    'end',
                    'template_set_balance_sheet',
                    'template_profit_loss_statement')
    list_display_links = ('title',
                          'begin',
                          'end',
                          'template_set_balance_sheet',
                          'template_profit_loss_statement')
    fieldsets = (
        (_('Basics'), {
            'fields': ('title',
                       'begin',
                       'end',
                       'template_set_balance_sheet',
                       'template_profit_loss_statement')
        }),
    )
    inlines = [InlineBookings, ]
    save_as = True

    form = AccountingPeriodForm

    def save_formset(self, request, form, formset, change):
        instances = formset.save(commit=False)
        for instance in instances:
            if change:
                instance.last_modified_by = request.user
            else:
                instance.last_modified_by = request.user
                instance.staff = request.user
            instance.save()

    def create_pdf_of_balance_sheet(self, request, queryset):
        from koalixcrm.crm.views.pdfexport import PDFExportView
        for obj in queryset:
            response = PDFExportView.export_pdf(self,
                                                request,
                                                obj,
                                                ("/admin/accounting/"+obj.__class__.__name__.lower()+"/"),
                                                obj.template_set_balance_sheet)
            return response

    create_pdf_of_balance_sheet.short_description = _("Create PDF of Balance Sheet")

    def create_pdf_of_profit_loss_statement(self, request, queryset):
        from koalixcrm.crm.views.pdfexport import PDFExportView
        for obj in queryset:
            response = PDFExportView.export_pdf(self,
                                                request,
                                                obj,
                                                ("/admin/accounting/"+obj.__class__.__name__.lower()+"/"),
                                                obj.template_profit_loss_statement,)
            return response

    create_pdf_of_profit_loss_statement.short_description = _("Create PDF of Profit Loss Statement Sheet")

    actions = ['create_pdf_of_balance_sheet',
               'create_pdf_of_profit_loss_statement']



from rest_framework import serializers

from koalixcrm.crm.reporting.work import Work
from koalixcrm.crm.reporting.human_resource import HumanResource
from koalixcrm.crm.reporting.task import Task
from koalixcrm.crm.reporting.reporting_period import ReportingPeriod
from koalixcrm.crm.rest.human_resource_rest import OptionHumanResourceJSONSerializer
from koalixcrm.crm.rest.reporting_period_rest import OptionReportingPeriodJSONSerializer
from koalixcrm.crm.rest.task_type_rest import OptionTaskJSONSerializer


class OptionWorkJSONSerializer(serializers.HyperlinkedModelSerializer):
    humanResource = OptionHumanResourceJSONSerializer(required=False, read_only=True)
    reportingPeriod = OptionReportingPeriodJSONSerializer(required=False, read_only=True)
    task = OptionTaskJSONSerializer(required=False, read_only=True)
    date = serializers.DateField(source='date')
    startTime = serializers.TimeField(source='start_time')
    stopTime = serializers.TimeField(source='stop_time')
    workedHours = serializers.DecimalField(source='worked_hours')
    shortDescription = serializers.CharField(source='short_description')
    description = serializers.CharField(source='description')

    class Meta:
        model = Work
        fields = ('human_resource',
                  'reporting_period',
                  'task',
                  'date',
                  'start_time',
                  'stop_time',
                  'worked_hours',
                  'short_description',
                  'description',)


class WorkJSONSerializer(serializers.HyperlinkedModelSerializer):
    humanResource = OptionHumanResourceJSONSerializer(source='human_resource')
    reportingPeriod = OptionReportingPeriodJSONSerializer(source='reporting_period',)
    task = OptionTaskJSONSerializer(source='task')
    date = serializers.DateField(source='date')
    startTime = serializers.TimeField(source='start_time')
    stopTime = serializers.TimeField(source='stop_time')
    workedHours = serializers.DecimalField(source='worked_hours')
    shortDescription = serializers.CharField(source='short_description')
    description = serializers.CharField(source='description')

    class Meta:
        model = Work
        fields = ('human_resource',
                  'reporting_period',
                  'task',
                  'date',
                  'start_time',
                  'stop_time',
                  'worked_hours',
                  'short_description',
                  'description',)

    def create(self, validated_data):
        work = Work()
        # Deserialize human_resource
        human_resource = validated_data.pop('human_resource')
        if human_resource:
            if human_resource.get('id', None):
                work.resource_type = HumanResource.objects.get(id=human_resource.get('id', None))
            else:
                work.resource_type = None
        # Deserialize reporting_period
        reporting_period = validated_data.pop('reporting_period')
        if reporting_period:
            if reporting_period.get('id', None):
                work.reporting_period = ReportingPeriod.objects.get(id=reporting_period.get('id', None))
            else:
                work.reporting_period = None
        # Deserialize task
        task = validated_data.pop('task')
        if task:
            if task.get('id', None):
                work.task = Task.objects.get(id=task.get('id', None))
            else:
                work.task = None

    def update(self, work, validated_data):
        # Deserialize human_resource
        human_resource = validated_data.pop('human_resource')
        if human_resource:
            if human_resource.get('id', work.resource_type):
                work.human_resource = HumanResource.objects.get(id=human_resource.get('id', None))
            else:
                work.human_resource = work.human_resource_id
        else:
            work.human_resource = None
        # Deserialize reporting_period
        reporting_period = validated_data.pop('reporting_period')
        if reporting_period:
            if reporting_period.get('id', work.resource_type):
                work.reporting_period = ReportingPeriod.objects.get(id=reporting_period.get('id', None))
            else:
                work.reporting_period = work.reporting_period_id
        else:
            work.reporting_period = None
        # Deserialize task
        task = validated_data.pop('task')
        if task:
            if task.get('id', work.task):
                work.task = Task.objects.get(id=task.get('id', None))
            else:
                work.task = work.task_id
        else:
            work.task = None
